package jp.agentec.sinaburocast.service;

import java.util.ArrayList;
import java.util.List;

import jp.agentec.sinaburocast.common.SinaburoConstant.MailSendType;
import jp.agentec.sinaburocast.entity.EnqueteMailSendResult;
import jp.agentec.sinaburocast.entity.NotifyMailSendResult;
import jp.agentec.sinaburocast.form.admin.mail.MailResultForm;

import org.seasar.extension.jdbc.AutoSelect;
import org.seasar.extension.jdbc.where.SimpleWhere;
import org.seasar.framework.container.annotation.tiger.Component;
import org.seasar.framework.container.annotation.tiger.InstanceType;

@Component(instance=InstanceType.SINGLETON)
public class EnqueteMailSendResultService extends AbstractService<EnqueteMailSendResult> {

	private static final String ID_SEQ_NAME = "enquete_mail_send_result_id_seq";

	public NotifyMailSendResultService notifyMailSendResultService;

    public EnqueteMailSendResult findById(Long enqueteMailSendResultId) {
        return select().id(enqueteMailSendResultId).getSingleResult();
    }

    public List<EnqueteMailSendResult> findAllOrderById() {
        return select().orderBy("enqueteMailSendResultId asc").getResultList();
    }

    public List<EnqueteMailSendResult> findAllOrderById(Integer enqueteMailId) {
        return select().where(new SimpleWhere().eq("enqueteMailId", enqueteMailId)).getResultList();
    }

    /**
     * 会員削除時、削除対象の会員検索
     * @param memberId
     * @return
     */
    public List<EnqueteMailSendResult> findAllOrderByIdList(Integer memberId) {
                return select().where(new SimpleWhere().eq("memberId", memberId)).getResultList();
    }

    /**
     * IDを発行して、登録する。
     * @param enqueteMailSendResult
     * @param insId
     * @return
     */
	public int insertEnqueteMail(EnqueteMailSendResult enqueteMailSendResult, String insId) {
		enqueteMailSendResult.enqueteMailSendResultId = getSeqNextVal(Long.class, ID_SEQ_NAME);

		int result = super.insert(enqueteMailSendResult, insId);

		return result;
	}

	/**
	 * メール送信結果詳細検索
	 * @param pageKey
	 * @param curPage
	 * @param mailResultForm
	 * @return
	 */
	public List<EnqueteMailSendResult> findAllOrderByIdResult(String pageKey, Integer curPage, MailResultForm mailResultForm) {
		List<EnqueteMailSendResult> enqueteMailSendResultList;

		if (mailResultForm.type == MailSendType.AUTO) {
			mailResultForm.setPaging(null, curPage, notifyMailSendResultService.getFindByMemberNotifyMailIdConut(Integer.parseInt(mailResultForm.enqueteMailId)));
			List<NotifyMailSendResult> notifyMailSendResultList = notifyMailSendResultService.findByMemberNotifyMailId(Integer.parseInt(mailResultForm.enqueteMailId), mailResultForm.getOffSet(pageKey), mailResultForm.getLimit(pageKey));
			enqueteMailSendResultList = new ArrayList<EnqueteMailSendResult>();

			for (NotifyMailSendResult result : notifyMailSendResultList) {
				enqueteMailSendResultList.add(exchangeEnqueteMailSendResult(result));
			}
		} else {
			AutoSelect<EnqueteMailSendResult> resultCnt = select().leftOuterJoin("member").where(new SimpleWhere().eq("enqueteMailId", mailResultForm.enqueteMailId));
			mailResultForm.setPaging(null, curPage, resultCnt.getCount());
			enqueteMailSendResultList = select().leftOuterJoin("member").where(new SimpleWhere().eq("enqueteMailId", mailResultForm.enqueteMailId)).orderBy("T1_.update_date desc").offset(mailResultForm.getOffSet(pageKey)).limit(mailResultForm.getLimit(pageKey)).getResultList();
		}

		return enqueteMailSendResultList;
	}

    /**
     * エラーレポート作成用メール送信結果詳細検索
     * @param enqueteMailId
     * @return
     */
	public List<EnqueteMailSendResult> findAllOrderByIdReport(Integer enqueteMailId, int type) {
		List<EnqueteMailSendResult> enqueteMailSendResultList;

		if (type == MailSendType.AUTO) {
			List<NotifyMailSendResult> notifyMailSendResultList = notifyMailSendResultService.findErrorByMemberNotifyMailId(enqueteMailId);
			enqueteMailSendResultList = new ArrayList<EnqueteMailSendResult>();

			for (NotifyMailSendResult result : notifyMailSendResultList) {
				enqueteMailSendResultList.add(exchangeEnqueteMailSendResult(result));
			}
		} else {
			enqueteMailSendResultList = select().leftOuterJoin("member").where(new SimpleWhere().eq("enqueteMailId", enqueteMailId).ne("sendResultCd", 0)).orderBy("T1_.member_id desc").getResultList();
		}
		return enqueteMailSendResultList;
	}

	/**
	 * メール送信エラーチェックバッチから
	 * @param messageId
	 * @return
	 */
	 public List<EnqueteMailSendResult> findMailByMessageId(String messageId) {
		return select().where(new SimpleWhere().eq("messageId", messageId)).getResultList();
	 }

	private EnqueteMailSendResult exchangeEnqueteMailSendResult(NotifyMailSendResult result) {
		EnqueteMailSendResult enqueteMailSendResult = new EnqueteMailSendResult();
		enqueteMailSendResult.enqueteMailId = result.memberNotifyMailId;
		enqueteMailSendResult.memberId = null;
		enqueteMailSendResult.email = result.email;
		enqueteMailSendResult.sendResultCd = result.sendResultCd;
		enqueteMailSendResult.sendResult = result.sendResult;
		enqueteMailSendResult.statusCd = result.statusCd;
		enqueteMailSendResult.status = result.status;
		enqueteMailSendResult.messageId = result.messageId;
		return enqueteMailSendResult;
	}
}